package mbs.controllers;



// EXTERNAL IMPORTS
import javax.microedition.broadcast.esg.Service;
import javax.microedition.broadcast.ServiceComponent;
import javax.microedition.broadcast.ServiceContext;
import javax.microedition.broadcast.ServiceContextListener;
import javax.microedition.media.Player;
import java.util.Vector;
import java.util.Enumeration;

/**
 * Controls the presentation of a broadcast service
 */
public class PresentationController implements ServiceContextListener
{
    // Singleton instance
    private static PresentationController sInstance;
    // Default service context
    private ServiceContext iServiceContext;
    // Selected service
    private Service iSelectedService;
    // Displayable for the video
    private Player[] iCurrentPlayers;
    // Vector of listeners
    private Vector iListeners;

    /**
     * From ServiceContextListener
     */
    public void contextUpdate(
        ServiceContext aContext,
        String aEvent,
        Object aData)
    {
        System.out.println("PresentationController.contextUpdate() - event: " + aEvent);
        System.out.println("PresentationController.contextUpdate() - aData: " + aData);

        // Use equalsIgnoreCase because RI has a bug...
        if (aEvent.equalsIgnoreCase(ServiceContextListener.PLAYERS_REALIZED))
        {
            // Presentation has been started but the controller does not
            // start showing the content. The client must obtain the players
            // and modify the VideoControl to display the content
            iCurrentPlayers = (Player[])aData;
            // Presentation will start soon
            notifyListeners(PresentationListener.EVENT_PRESENTING, null);
        }
    }

    /**
     * Returns the singleton instance of this class
     */
    public static PresentationController getInstance()
    {
        if (sInstance == null)
        {
            sInstance = new PresentationController();
        }

        return sInstance;
    }

    /**
     * Adds a new presentation listener to this object
     *
     * @param aListener The listener to be added
     */
    public void addListener(PresentationListener aListener)
    {
        if (!iListeners.contains(aListener))
        {
            iListeners.addElement(aListener);
        }
    }

    /**
     * Removes an existing presentation listener from this object
     *
     * @param aListener The listener to be removed
     */
    public void removeListener(PresentationListener aListener)
    {
        iListeners.removeElement(aListener);
    }

    /**
     * Returns a list of players which are presenting the current service
     *
     * @param aComponent The service component which player is requested
     * @return The player persenting the specified service component
     */
    public Player getPlayer(ServiceComponent aComponent)
    {
        checkPrepared();

        Player player = null;

        try
        {
            iServiceContext.getPlayer(aComponent);
        }
        catch (Exception e)
        {
            System.out.println("PresentationController.getPlayer() - error: "
                               + e.toString());
        }

        return player;
    }

    /**
     * Returns a list of current players presenting the media
     */
    public Player[] getCurrentPlayers()
    {
        checkPrepared();

        return iCurrentPlayers;
    }

    /**
     * Returns the default player
     */
    public Player getDefaultPlayer()
    {
        checkPrepared();

        return iCurrentPlayers[ 0 ];
    }

    /**
     * Returns the list of selected service components
     *
     * @return The list of selected service component
     */
    public ServiceComponent[] getSelectedComponents()
    {
        checkPrepared();

        return iServiceContext.getSelectedComponents();
    }

    /**
     * Returns the list of all service components
     *
     * @return The list of all service component
     */
    public ServiceComponent[] getAllComponents()
    {
        checkPrepared();
        return iServiceContext.getAllComponents();
    }

    /**
     * Prepares the presentation controller. This function does not start
     * the presentation yet
     *
     * @param aService Service which will be presented when the presentation
     *        is started
     */
    public void prepare(Service aService)
    {
        // Get first service guide
        iServiceContext = ServiceContext.getDefaultContext();
        iServiceContext.addListener(this);
        iSelectedService = aService;
    }

    /**
     * Starts the presentation of the selected broadcast service
     *
     * @param aDisplayable The component which is used to display the service
     */
    public void start()
    {
        checkPrepared();

        try
        {
            // Select the service
            iServiceContext.select(iSelectedService);
        }
        catch (Exception e)
        {
            throw new IllegalStateException("Presentation controller not prepared");
        }
    }

    /**
     * Stops the presentation of the selected service. Nothing happens if the
     * Service controller has not been prepared or is in closed state
     */
    public void stop()
    {
        if (iServiceContext != null &&
                iServiceContext.getState() != ServiceContext.STOPPED)
        {
            iServiceContext.stop();
        }
    }

    /**
     * Returns the currently selected service
     */
    public Service getCurrentService()
    {
        return iSelectedService;
    }

    /**
     * Returns the signal quality of this service context
     */
    public int getSignalQuality()
    {
        checkPrepared();

        return iServiceContext.getSignalQuality();
    }

    /**
     * Notifies all registered listeners
     *
     * @param aEventType Type of the event to be delivered to the listeners
     * @param aMessage Message which will be delivered to the listeners
     */
    private void notifyListeners(int aEventType, String aMessage)
    {
        Enumeration listeners = iListeners.elements();
        while (listeners.hasMoreElements())
        {
            PresentationListener listener =
                (PresentationListener)listeners.nextElement();
            listener.notifyPresentationEvent(aEventType, aMessage);
        }
    }

    /**
     * Checks if this presentation controller object is prepared or not
     */
    private void checkPrepared()
    {
        if (iServiceContext == null)
        {
            throw new IllegalStateException("Presentation controller not prepared");
        }
    }

    /**
     * Constructor
     */
    private PresentationController()
    {
        iListeners = new Vector();
    }

}

// End of File
