package mbs.controllers;



// EXTERNAL IMPORTS
import javax.microedition.broadcast.esg.*;
import java.util.Vector;
import java.util.Date;

/**
 * Controls the electronic service guide
 */
public class ServiceGuideController
{
    public static final int QUERY_SERVICES = 0;
    public static final int QUERY_PROGRAMS = 1;

    // Singleton instance
    private static ServiceGuideController sInstance;
    // Current Service guide
    private ServiceGuide iServiceGuide;
    // Current query
    private Query iQuery;
    // Query type
    private int iQueryType;
    // Sorting keys
    private Vector iSortKeys = new Vector();
    // Currently selected service
    private Service iSelectedService;
    // Query result
    private Vector iQueryResult;

    /**
     * Returns the singleton instance of this class
     */
    public static ServiceGuideController getInstance()
    {
        if (sInstance == null)
        {
            sInstance = new ServiceGuideController();
        }

        return sInstance;
    }

    /**
     * Initializes the service guide controller
     */
    public synchronized void initialize()
    {
        if (iServiceGuide == null)
        {
            // Get first service guide
            iServiceGuide = ServiceGuide.getAllServiceGuides()[ 0 ];

            iQuery = null;
            System.out.println("ServiceGuideController.initialize: size = " + iSortKeys.size());
        }
    }

    /**
     * De-initializes the service guide controller
     */
    public synchronized void deinitialize()
    {
        iServiceGuide = null;
    }

    /**
     * Sets the executed query
     *
     * @param aQuery The query to be executed
     * @param aSortKeys Keys which are used in sorting the result
     * @param aQueryType The type of the query
     */
    public void setQuery(Query aQuery, int aQueryType)
    {
        iQueryType = aQueryType;
        iQuery = aQuery;
    }

    /**
     * Returns the current query. Null is returned if no query has been set
     */
    public Query getQuery()
    {
        return iQuery;
    }

    /**
     * Selects a service
     */
    public void setService(Service aService)
    {
        iSelectedService = aService;
    }

    /**
     * Gets the selected service
     */
    public Service getService()
    {
        return iSelectedService;
    }

    /**
     * Sets the keys used to sort the query results
     *
     * @param aSortKeys Keys which are used in sorting the result
     */
    public void setSortKeys(Vector aSortKeys)
    {
        if (aSortKeys != null)
        {
            iSortKeys = aSortKeys;
            System.out.println("ServiceGuideController.setSortKeys: size = " + iSortKeys.size());
        }
    }

    /**
     * Resets query result sorting keys
     */
    public void resetSortKeys()
    {
        iSortKeys = new Vector();
        System.out.println("ServiceGuideController.resetSortKeys: size = " + iSortKeys.size());
    }

    /**
     * Executes the specified query
     */
    public Vector executeQuery()
    {
        checkValidity();

        try
        {
            int size = iSortKeys.size();
            System.out.println("ServiceGuideController.executeQuery: size = " + size);
            Attribute[] sortAttribs = null;
            if (size > 0)
            {
                sortAttribs = new Attribute[ size ];
                iSortKeys.copyInto(sortAttribs);
            }

            if (iQueryType == QUERY_SERVICES)
            {
                // Query may be null which indicates that all services are returned
                Service[] queryResult =
                    iServiceGuide.findServices(iQuery, sortAttribs, -1, -1);
                int count = queryResult.length;
                iQueryResult = new Vector(count);
                // Copy query result
                for (int i = 0; i < queryResult.length; i++)
                {
                    iQueryResult.addElement(queryResult[ i ]);
                }
            }
            else if (iQueryType == QUERY_PROGRAMS)
            {
                // Query may be null which indicates that all programs are returned
                ProgramEvent[] queryResult =
                    iServiceGuide.findPrograms(iQuery, sortAttribs, -1, -1);
                int count = queryResult.length;
                iQueryResult = new Vector(count);
                // Copy query result
                for (int i = 0; i < queryResult.length; i++)
                {
                    iQueryResult.addElement(queryResult[ i ]);
                }
            }
        }
        catch (QueryException e)
        {
            throw new RuntimeException("QueryException: " + e.getMessage());
        }

        return iQueryResult;
    }

    /**
     * Updates the service guide
     */
    public void updateServiceGuide()
    {
        checkValidity();
        iServiceGuide.forceUpdate();
    }

    /**
     * Adds a new listener to the current service guide
     *
     * @param aListener The listener to be registered
     * @throws IllegalStateException if the service guide controller has
     *         not been initialized
     */
    public void addListener(ServiceGuideListener aListener)
    {
        checkValidity();
        try
        {
            iServiceGuide.addListener(aListener, null);
        }
        catch (QueryException e)
        {
            // Ignore at this point
        }
    }

    /**
     * Removes an existing listener from the current service guide
     *
     * @param aListener The listener to be removed
     * @throws IllegalStateException if the service guide controller has
     *         not been initialized
     */
    public void removeListener(ServiceGuideListener aListener)
    {
        checkValidity();
        iServiceGuide.removeListener(aListener);
    }

    /**
     * Gets the last update time for the associated service guide
     *
     * @return The last updated time of the associated service guide
     * @throws IllegalStateException if the service guide controller has
     *         not been initialized
     */
    public Date getUpdateTime()
    {
        checkValidity();
        return iServiceGuide.getLastUpdatedTime();
    }

    /**
     * Checks whether this service guide controller has been initialized or not
     *
     * @throws IllegalStateException if the service guide controller has
     *         not been initialized
     */
    private void checkValidity()
    {
        if (iServiceGuide == null)
        {
            throw new IllegalStateException("Service guide not initialized");
        }
    }

    /**
     * Constructor
     */
    private ServiceGuideController()
    {
    }
}

// End of File
