package mbs.views;


// EXTERNAL IMPORTS
import javax.microedition.lcdui.*;
import javax.microedition.broadcast.esg.*;
import java.util.Vector;
import java.util.Enumeration;
import mbs.controllers.ServiceGuideController;
import mbs.controllers.SubscriptionController;
/**
 * Shows information about program events
 *
 * Events can be either from currently selected service
 * or from a query specified by the user
 */
public class ProgramEventsView extends List implements ViewBase, ServiceGuideListener
{
    // Service guide controller
    private ServiceGuideController iController;
    // List of programs
    private Vector iPrograms;
    // Commands
    private Command iCmdBack;
    private Command iCmdInfo;
    private Command iCmdListPurchaseObjects;

    private Initializer iInitializer;

    // Subscription controller
    private SubscriptionController iSubscriptionController;

    /**
     * Constructor
     */
    ProgramEventsView()
    {
        super("Program Events", List.IMPLICIT);

        iCmdBack = new Command("Back", Command.BACK, 1);
        iCmdInfo = new Command("Show details", Command.SCREEN, 1);
        iCmdListPurchaseObjects = new Command("Get purchase objects", Command.SCREEN, 1);

        addCommand(iCmdBack);
        addCommand(iCmdInfo);
        addCommand(iCmdListPurchaseObjects);

        iController = ServiceGuideController.getInstance();
        iPrograms = new Vector();

        iInitializer = new Initializer();
        iInitializer.start();
    }

    /**
     * From ServiceGuideListener
     */
    public void serviceGuideUpdated(
        String aEvent,
        ServiceGuideData aServiceGuideData,
        Object aEventData)
    {
        if (aEvent.equalsIgnoreCase(ServiceGuideListener.PROGRAM_DELETED))
        {
            synchronized (this)
            {
                removeProgram((ProgramEvent)aServiceGuideData);
            }
        }
        else if (aEvent.equalsIgnoreCase(ServiceGuideListener.SERVICE_GUIDE_BULK_CHANGED) ||
                 aEvent.equalsIgnoreCase(ServiceGuideListener.NEW_PROGRAM_LISTED))
        {
            requestUpdate();
        }
        else if (aEvent.equalsIgnoreCase(ServiceGuideListener.SERVICE_GUIDE_DELETED))
        {
            synchronized (this)
            {
                iPrograms.removeAllElements();
                deleteAll();
            }
        }
    }

    /**
     * Activates the view
     */
    public void activate()
    {
        iSubscriptionController = SubscriptionController.getInstance();

        iPrograms.removeAllElements();
        deleteAll();

        requestUpdate();
        iController.addListener(this);
    }

    /**
     * Deactivates the view
     */
    public void deactivate()
    {
        iController.removeListener(this);
    }

    /**
     * Returns the displayable of this view
     *
     * @return Displayable of this view
     */
    public Displayable getDisplayable()
    {
        return this;
    }

    /**
     * Sets the command listener for this view
     *
     * @param aListener The command listener listening commands for this view
     */
    public void setCommandListener(CommandListener aListener)
    {
        super.setCommandListener(aListener);
    }

    /**
     * Handles a command which is delegated to this object
     *
     * @param aCommand The Command to be handled
     * @param aDisplayable The displayable which initiated the command
     */
    public void handleCommand(Command aCommand, Displayable aDisplayable)
    {
        if (aCommand == iCmdBack)
        {
            ViewHandler.activateView(ViewHandler.PLATFORM_PROVIDER_VIEW_ID);
            return;
        }

        int index = getSelectedIndex();
        if (index < 0)
        {
            return;
        }

        ProgramEvent program = (ProgramEvent)iPrograms.elementAt(index);

        if (aCommand == iCmdInfo)
        {
            new ServiceGuideDataInfo(getString(index), this, program);
        }
        else if (aCommand == iCmdListPurchaseObjects)
        {
            String programId;
            try
            {
                programId = (String)program.getStringValue(
                                CommonMetadataSet.PROGRAM_ID);
            }
            catch (DataUnavailableException due)
            {
                due.printStackTrace();
                return;
            }

            iSubscriptionController.setId(programId);
            ViewHandler.activateView(ViewHandler.PURCHASE_OBJECTS_VIEW_ID);
        }
    }

    private synchronized void requestUpdate()
    {
        // Notify updater if not running
        try
        {
            synchronized (iInitializer)
            {
                iInitializer.notify();
            }
        }
        catch (Exception e)
        {
        }
    }

    private synchronized void removeProgram(ProgramEvent aProgram)
    {
        int index = iPrograms.indexOf(aProgram);
        if (index != -1)
        {
            iPrograms.removeElement(aProgram);
            delete(index);
        }
    }

    /**
     * Updates the list of program events
     */
    private synchronized void updatePrograms(Vector aPrograms)
    {
        // Check withc services have been removed
        Enumeration currentPrograms = iPrograms.elements();
        while (currentPrograms.hasMoreElements())
        {
            ProgramEvent program = (ProgramEvent)currentPrograms.nextElement();
            if (!aPrograms.contains(program))
            {
                removeProgram(program);
            }
        }

        // Check that which services are new
        Enumeration newPrograms = aPrograms.elements();
        while (newPrograms.hasMoreElements())
        {
            ProgramEvent program = (ProgramEvent)newPrograms.nextElement();
            if (!iPrograms.contains(program))
            {
                addProgram(program);
            }
        }
    }

    /**
     * Adds new program event to the list
     */
    public synchronized void addProgram(ProgramEvent aProgram)
    {
        iPrograms.addElement(aProgram);
        try
        {
            append(aProgram.getStringValue(CommonMetadataSet.PROGRAM_NAME), null);
        }
        catch (DataUnavailableException due)
        {
            append("(Unnamed)", null);
        }

        int index = iPrograms.size() - 1;
        setFont(index, ViewUtils.getDefaultFont());
    }

    /**
     * Inner class which is used for service list initialization
     */
    private class Initializer extends Thread
    {
        /**
         * Constructor
         *
         * @param aUpdateMode Does a full update if true.
         */
        public Initializer()
        {
        }

        /**
         * Run method for this thread
         */
        public void run()
        {
            while (true)
            {
                try
                {
                    synchronized (this)
                    {
                        this.wait();
                    }
                    // Get selected service
                    Service service = iController.getService();
                    Vector programEvents = null;

                    if (service != null)
                    {
                        ProgramEvent[] programs = service.getProgramEvents();
                        int count = programs.length;
                        programEvents = new Vector(count);
                        for (int i = 0; i < count; i++)
                        {
                            programEvents.addElement(programs[ i ]);
                        }
                    }
                    else
                    {
                        programEvents = iController.executeQuery();
                    }

                    if (programEvents != null)
                    {
                        updatePrograms(programEvents);
                    }
                }
                catch (Exception e)
                {
                    // Interrupted or service guide invalidated
                    e.printStackTrace();
                }
            }
        }
    }
}

// End of File
