package mbs.views;

// EXTERNAL IMPORTS
import java.util.Hashtable;
import java.util.Vector;
import javax.microedition.midlet.MIDlet;
import javax.microedition.lcdui.*;

/**
 * Handles the view activation and command dispatching
 * in the application
 */
public class ViewHandler implements CommandListener {
    // Public view identifiers

    public static final int MAIN_VIEW_ID = 0;
    public static final int PLATFORM_VIEW_ID = 1;
    public static final int PLATFORM_PROVIDER_VIEW_ID = 2;
    public static final int SERVICES_VIEW_ID = 3;
    public static final int PROGRAMS_VIEW_ID = 4;
    public static final int QUERY_COMPOSER_VIEW_ID = 5;
    public static final int VIDEO_FORM_VIEW_ID = 6;
    public static final int VIDEO_CANVAS_VIEW_ID = 7;
    public static final int SORT_ATTRIBUTES_VIEW_ID = 8;
    public static final int SERVICE_COMPONENTS_VIEW_ID = 9;
    public static final int PURCHASE_OBJECTS_VIEW_ID = 10;
    public static final int SUBSCRIPTION_VIEW_ID = 11;
    public static final int PRICE_OPTION_VIEW_ID = 12;
    public static final int SUBSCRIBED_OBJECTS_VIEW_ID = 13;
    // Vector containing views
    private Hashtable iViews;
    private ViewBase iCurrentView;
    // Previous views are added to the vector when the user selects
    // a new view. The views are removed when the user goes back
    // to the previous views.
    private Vector iPreviousViews;
    private Display iDisplay;
    private MIDlet iMIDlet;
    // Singleton instance of this view handler
    private static ViewHandler sInstance;

    public static ViewHandler getInstance() {
        if (sInstance == null) {
            sInstance = new ViewHandler();
        }
        return sInstance;
    }

    /**
     * Constructor
     */
    private ViewHandler() {
        iViews = new Hashtable();
        iPreviousViews = new Vector(5);

        // Create all view used in the application
        iViews.put(new Integer(PLATFORM_VIEW_ID), new PlatformView());
        iViews.put(new Integer(PLATFORM_PROVIDER_VIEW_ID), new PlatformProviderView());
        iViews.put(new Integer(PROGRAMS_VIEW_ID), new ProgramEventsView());
        iViews.put(new Integer(SERVICES_VIEW_ID), new ServicesView());
        iViews.put(new Integer(VIDEO_CANVAS_VIEW_ID), new VideoCanvasView());
        iViews.put(new Integer(PURCHASE_OBJECTS_VIEW_ID), new PurchaseObjectsView());
        iViews.put(new Integer(SUBSCRIPTION_VIEW_ID), new SubscriptionView());
        iViews.put(new Integer(PRICE_OPTION_VIEW_ID), new PriceOptionView());
        iViews.put(new Integer(SUBSCRIBED_OBJECTS_VIEW_ID), new SubscribedObjectsView());
    }

    /**
     * Sets the display of the MIDlet to this view handler
     * @param aDisplay The display of the MIDlet
     */
    public void setDisplay(Display aDisplay) {
        iDisplay = aDisplay;
    }

    /**
     * Sets the current MIDlet to this view handler
     * The MIDlet instance is needed to exit from
     * the application
     * @param aMIDlet Current running MIDlet
     */
    public void setMIDlet(MIDlet aMIDlet) {
        iMIDlet = aMIDlet;
    }

    public MIDlet getMIDlet() {
        return iMIDlet;
    }

    /**
     * Switches view to the requested view
     * @param aViewId View id which is to be activated
     */
    public static void activateView(int aViewId) {
        ViewHandler handler = ViewHandler.getInstance();
        ViewBase view = (ViewBase) handler.iViews.get(new Integer(aViewId));
        handler.doSwitchView(view);
    }

    /**
     * Switches view to the pervious view
     * @param aViewId View id which is to be activated
     */
    public static void activatePreviousView() {
        ViewHandler handler = ViewHandler.getInstance();
        // Activate previous view (=null)
        handler.doSwitchView(null);
    }

    /**
     * Returns the display to which the application draws
     * @return The display to which the application draws
     */
    public static Display getDisplay() {
        return getInstance().iDisplay;
    }

    /**
     * Returns the current view.
     */
    public static ViewBase getCurrentView() {
        return getInstance().iCurrentView;
    }

    /**
     * commandAction from CommandListener
     */
    public void commandAction(Command aCommand, Displayable aDisp) {
        if (aCommand.getCommandType() == Command.EXIT) {
            iMIDlet.notifyDestroyed();
        } else {
            // Catch exceptions here
            try {
                iCurrentView.handleCommand(aCommand, aDisp);
            } catch (Exception e) {
                e.printStackTrace();
                showAlert(e);
            }
        }
    }

    /**
     * Switches to the specified view.
     *
     * @param aView View to be activated or null if the previous
     *        view should be selected.
     */
    private void doSwitchView(ViewBase aView) {

        // Deactivate the current view.
        if (iCurrentView != null) {
            iCurrentView.deactivate();
        }

        // null view means that the previous view should be activated.
        if (aView == null) {
            if (iPreviousViews.size() > 0) {
                iCurrentView = (ViewBase) iPreviousViews.lastElement();
                iPreviousViews.removeElement(iCurrentView);
            }
        } else {
            if (iCurrentView != null) {
                iPreviousViews.addElement(iCurrentView);
            }

            iCurrentView = aView;
        }

        // Switch view if it is available
        if (iCurrentView != null) {
            iCurrentView.setCommandListener(this);
            iCurrentView.activate();
            iDisplay.setCurrent(iCurrentView.getDisplayable());
        }


    }

    /**
     * Shows alert note
     */
    private void showAlert(Exception aException) {
        AlertNote alert = new AlertNote("Exception",
                aException.toString(),
                AlertType.ERROR,
                iCurrentView.getDisplayable());
        alert.execute();
    }
}
// End of File

